/* screenio.c - 16:00 GMT +10:00 Wed 4 Aug 1993 - modifier Geoffrey Tobin */

/* From input file "../include/globals.p" */

#include "config.h"
#include "globals.h"
#include "screenio.h"
#include "vdu.h"
#include <ggi/ggi.h>

/* Variables Exported, via "screenio.h" */

Char TeXtoASCII[XXX];
extern ggi_visual_t *GGIvisual;
extern int argument;
void InitGGI(void);

/* SYSDEP: We need to use the Unix write routine.
   In particular, using Pascal's write/writeln has some nasty side-effect
   that prevents BusyRead from working!
*/

int textlinewidth;    /* text characters (columns) per line */
int textcolumn = 0;   /* column number in current text line */

/******************************************************************************/

int MesgChar (int ch)
{
  int status = 0;
  /* maxcol <= both size of a string, and width of a line */
  int maxcol = (maxstring < textlinewidth ? maxstring : textlinewidth);

  /* Keep messages within maxcol columns */
  if (textcolumn >= maxcol)
    status = EOF;
  else
  {
      WriteTextChar (ch);
    ++ textcolumn;
  }
#ifdef COLUMN_LOG
#endif
  return status;
}
/* MesgChar */

/******************************************************************************/

#ifdef __STDC__
int MesgString (const char * s)
#else
int MesgString (s)
  char * s;
#endif
{
  int i, status = 0;
  string message;
  /* maxcol <= both size of a string, and width of a line */
  int maxcol = (maxstring < textlinewidth ? maxstring : textlinewidth);

  /* Keep messages within maxcol columns */
  if (textcolumn < 0 || textcolumn >= maxcol)
  {
    strcpy (message, "");
  }
  else
  {
    strncpy (message, s, maxcol-textcolumn);
    message[maxcol-textcolumn] = '\0';
  }

    for (i = 0; message[i] != '\0'; i++)
      WriteTextChar (message[i]);


  textcolumn += strlen (message);
#ifdef COLUMN_LOG
#endif
  return status;
}
/* MesgString */

/******************************************************************************/

int MesgFlush (VOID)
{
  int status = fflush (stdout);
  textcolumn = 0;

  return status;
}
/* MesgFlush */

/******************************************************************************/

Void MesgLine (VOID)
{
    WriteTextChar (CR);
  MesgFlush();   /* MesgLine also updates terminal */
}
/* MesgLine */

/******************************************************************************/

#ifdef __STDC__
Static int MesgCard (int c)
#else
Static int MesgCard (c)
    int c;
#endif
{
  /* Since the vast majority of given values will be small numbers, we avoid
     recursion until c >= 100.
  */

  int len = 0;
  if (c < 10)
  {
    MesgChar (c + '0');
    ++ len;
  }
  else if (c < 100)
  {
    MesgChar (c / 10 + '0');
    MesgChar (c % 10 + '0');
    len += 2;
  }
  else  /* c >= 100 */
  {
    len += MesgCard (c / 100);   /* recursive if c >= 100 */
    c %= 100;
    MesgChar (c / 10 + '0');
    MesgChar (c % 10 + '0');
    len += 2;
  }
  return (len);
}
/* MesgCard */

/******************************************************************************/

int MesgInt (int i)
{
  int len = 0;
  if (i < 0)
  {
    MesgChar ('-');
    ++ len;
    i = abs (i);
  }
  len += MesgCard (i);
  return (len);
}
/* MesgInt */

/******************************************************************************/

Void WriteLine (VOID)
{
  WriteChar (CR);
  WriteFlush();   /* WriteLine also updates terminal */
}
/* WriteLine */

/******************************************************************************/

#ifdef __STDC__
Static Void WriteCard (int c)
#else
Static Void WriteCard (c)
    int c;
#endif
{
  /* Since the vast majority of given values will be small numbers, we avoid
     recursion until c >= 100.
  */

  if (c < 10)
  {
    WriteChar (c + '0');
    return;
  }
  if (c < 100)
  {
    WriteChar (c / 10 + '0');
    WriteChar (c % 10 + '0');
    return;
  }
  else
  {
    WriteCard (c / 100);   /* recursive if c >= 100 */
    c %= 100;
    WriteChar (c / 10 + '0');
    WriteChar (c % 10 + '0');
  }
}
/* WriteCard */

/******************************************************************************/

#ifdef __STDC__
Void WriteInt (int i)
#else
Void WriteInt (i)
    int i;
#endif
{
  if (i < 0)
  {
    WriteChar ('-');
    i = abs (i);
  }
  WriteCard (i);
}
/* WriteInt */

/******************************************************************************/

static Void Restore_Temp_Terminal (VOID)
{
  /* Restore_Temp_Terminal is for screenio.c only. */

  MesgFlush();   /* make sure terminal is up-to-date */
}
/* Restore_Temp_Terminal */

/******************************************************************************/

Void RestoreTerminal (VOID)
{
  /* RestoreTerminal restores the initial terminal setting. */
  /* It should be called before any client module terminates. */

  MesgFlush();   /* make sure terminal is up-to-date */
}
/* RestoreTerminal */

/******************************************************************************/

Void BreakGraphics (VOID)
{
    StartText();
    ClearScreen();
    MesgLine();
    ResetVDU();
    Restore_Temp_Terminal();
  }

Void RestoreGraphics (VOID)
{
    InitGGI();
    StartText();
    ClearScreen();
}

int ReadCommand(void)
{
    int result='0';
    argument=0;
    ggiFlush(GGIvisual);
    while(isdigit(result))
    {
        argument*=10;
        argument+=(result-'0');
        result=ggiGetc(GGIvisual);
    }
    return result;
}

/******************************************************************************/

Static Void InitTeXtoASCII (VOID)
{
  /* Initialize TeXtoASCII array used in specific ShowChar/Rectangle routines
     to map a given TeX char into a similar, displayable ASCII char.
  */

  int i;

  for (i = 0; i <= 0xa; i++)   /* Greek letters */
    TeXtoASCII[i] = '?';
  for (i = 0xb; i <= 0xf; i++)   /* ligatures */
    TeXtoASCII[i] = '?';
  TeXtoASCII[0x10] = 'i';   /* dotless i */
  TeXtoASCII[0x11] = 'j';   /* dotless j */
  TeXtoASCII[0x12] = '`';   /* grave accent */
  TeXtoASCII[0x13] = '\'';   /* acute accent */
  for (i = 0x14; i <= 0x17; i++)   /* high accents */
    TeXtoASCII[i] = '~';
  TeXtoASCII[0x18] = ',';   /* cedilla */
  for (i = 0x19; i <= 0x20; i++)   /* foreigns */
    TeXtoASCII[i] = '?';
  for (i = 0x21; i <= 0x5b; i++)   /* same */
    TeXtoASCII[i] = i;
  TeXtoASCII[0x5c] = '"';   /* open dble quote */
  TeXtoASCII[0x5d] = ']';   /* same */
  for (i = 0x5e; i <= 0x5f; i++)   /* more accents */
    TeXtoASCII[i] = '^';
  for (i = 0x60; i <= 0x7a; i++)   /* same */
    TeXtoASCII[i] = i;
  for (i = 0x7b; i <= 0x7c; i++)   /* en & em dash */
    TeXtoASCII[i] = '-';
  for (i = 0x7d; i <= 0x7f; i++)   /* more accents */
    TeXtoASCII[i] = '~';
  for (i = 0x80; i <= 0xff; i++)   /* non-ASCII, but 8-bit: 128 to 255 */
    TeXtoASCII[i] = '?';
}
/* InitTeXtoASCII */

/******************************************************************************/

Void InitScreenIO (VOID)
{
  InitTeXtoASCII();
  if (setvbuf (stdout, NULL, _IOFBF, BUFSIZ/4) != 0)
  {
    MesgString ("screenio.c: InitScreenIO: setvbuf failed!");
    MesgLine();
  }
}
/* InitScreenIO */

/*****************************************************************************/

Void FatalError (const char * s)
{
  StartText();
  ResetVDU();  /* let message stay */

  MesgString (s);
  MesgLine();

  RestoreTerminal();
  exit (1);
}

/*****************************************************************************/
/* end screenio.c */
